SystemOrganization addCategory: #'Puzzles-GUI'!
SystemOrganization addCategory: #'Puzzles-Core'!
SystemOrganization addCategory: #Puzzles!

Matrix subclass: #IrregularGrid
	instanceVariableNames: 'irregularPuzzleGui regions solver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!IrregularGrid class methodsFor: 'instance creation' stamp: 'mg 7/1/2014 15:26'!
newConstrainedPuzzleFromFile: aFileName

	| puzzle |
	puzzle := self newFromFile: aFileName.
	puzzle createConstraints.
	puzzle irregularPuzzleGui openInWorld.
	^ puzzle! !

!IrregularGrid class methodsFor: 'instance creation' stamp: 'mg 6/30/2014 13:19'!
newFromFile: aFileName

	| file charList grid puzzleSize |
	charList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:char | charList add: char]].
	puzzleSize := charList size sqrt.
	file close.
	grid := IrregularGrid new: puzzleSize element: 0.
	grid buildRegionsFrom: charList.
	grid buildGUI.

	^ grid! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 7/1/2014 17:37'!
allRegions

	| idRegionDict |
	idRegionDict := Dictionary new.
	self regions withIndicesDo: [:val :row :column |
		(idRegionDict 
			at: val 
			ifAbsent: [idRegionDict at: val put: OrderedCollection new]) 
		add: (self at: row at: column)].
	^ idRegionDict values! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:16'!
bottomNeighborOfX: x y: y

	x = self rowCount ifTrue: [^'' asCharacter].
	^ self regions at: x+1 at: y
	! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 17:32'!
buildGUI

	| borderLeft borderTop borderBottom borderRight |

	irregularPuzzleGui setMatrixCoordinatesOf: self.

	self regions withIndicesDo: [:val :row :column | | irregularCell | 
		borderTop := (self topNeighborOfX: row y: column) ~= val.
		borderBottom := (self bottomNeighborOfX: row y: column) ~= val.
		borderLeft := (self leftNeighborOfX: row y: column) ~= val.
		borderRight := (self rightNeighborOfX: row y: column) ~= val.

		irregularCell := irregularPuzzleGui cellAtX: row Y: column.
		borderTop ifTrue: [irregularCell addTopBorder].
		borderBottom ifTrue: [irregularCell addBottomBorder].
		borderLeft ifTrue: [irregularCell addLeftBorder].
		borderRight ifTrue: [irregularCell addRightBorder]].
	! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 6/27/2014 11:19'!
buildRegionsFrom: aCharacterList

	regions := Matrix new: self rowCount tabulate: [:i :j | aCharacterList at: (j \\ 10 + (i-1*9))].! !

!IrregularGrid methodsFor: 'initialize-release' stamp: 'mg 7/1/2014 13:39'!
initialize

	super initialize.
	regions := Matrix new: 9.
	irregularPuzzleGui := IrregularGridGUI new.
	solver := ConstraintSolver newZ3Solver.! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 14:00'!
irregularPuzzleGui

	^ irregularPuzzleGui! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:15'!
leftNeighborOfX: x y: y

	y = 1 ifTrue: [^'' asCharacter].
	^ self regions at: x at: y-1
	! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/27/2014 11:26'!
regionWithIdentifier: aCharacter

	| region |
	region := OrderedCollection new.
	regions withIndicesDo: [:val :row :col | 
		val = aCharacter ifTrue: [region add: (self at: row at: col)]].
	^ region! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/27/2014 11:35'!
regions

	^ regions! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:16'!
rightNeighborOfX: x y: y

	y = self columnCount ifTrue: [^'' asCharacter].
	^ self regions at: x at: y+1
	! !

!IrregularGrid methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:16'!
topNeighborOfX: x y: y

	x = 1 ifTrue: [^'' asCharacter].
	^ self regions at: x-1 at: y
	! !

IrregularGrid subclass: #StarbattleGrid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-Core'!

!StarbattleGrid class methodsFor: 'instance creation' stamp: 'mg 6/30/2014 17:33'!
newFromFile: aFileName

	| file charList grid puzzleSize |
	charList := OrderedCollection new.
	file := MultiByteFileStream fileNamed: aFileName.
	file upToEnd asString substrings
		collect: [:row | row do: [:char | charList add: char]].
	puzzleSize := charList size sqrt.
	file close.
	grid := StarbattleGrid new: puzzleSize element: 0.
	grid buildRegionsFrom: charList.
	grid buildGUI.

	^ grid! !

!StarbattleGrid methodsFor: 'solving' stamp: 'mg 7/2/2014 10:51'!
createConstraints
	"This method creates all necessary constraints to solve a Starbattle puzzle with Z3."
	solver variableType: 'Int'.
	solver autoSolve: false.

	"[(self regionWithIdentifier: $a) anySatisfy: [:cell | cell = true]] alwaysSolveWith: solver."
	"[(self containsStarAtX: 1 y: 1) = true] alwaysSolveWith: solver."
	"[(self atRow: 1) count: [:each | each = true] result: 2] alwaysSolveWith: solver."

	"1 means star, 0 means no star."
	[self allSatisfy: [:val | val between: 0 and: 1]] alwaysSolveWith: solver.

	"Each row, column and region contains exactly two stars."
	(1 to: self rowCount) do: [:index |
		[((self atRow: index) sum = 2) &
		((self atColumn: index) sum = 2)] alwaysSolveWith: solver].

	[self allRegions allSatisfy: [:region | region sum = 2]] alwaysSolveWith: solver.
	"[(self regionWithIdentifier: $a)] alwaysSolveWith: solver."

	"Stars cannot touch each other, not even diagonally."
	self withIndicesDo: [:val :row :column | [(self at: row at: column) = 0 | ((self neighborsOfX: row y: column) sum = 0)] alwaysSolveWith: solver].

	Transcript show: 'milliseconds to solve the sudoku constraints: ', [solver solve] timeToRun; cr.! !

!StarbattleGrid methodsFor: 'accessing' stamp: 'mg 7/1/2014 16:14'!
endlessBorderAt: x at: y
	"All cells outside the matrix are 0."
	x < 1 ifTrue: [^nil].
	y < 1 ifTrue: [^nil].
	x > nrows ifTrue: [^nil].
	y > ncols ifTrue: [^nil].
	^ self at: x at: y! !

!StarbattleGrid methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 17:33'!
initialize

	super initialize.
	irregularPuzzleGui := StarbattleGridGUI new.! !

!StarbattleGrid methodsFor: 'accessing' stamp: 'mg 7/1/2014 16:26'!
neighborsOfX: x y: y

	| result |
	result := OrderedCollection new.
	(-1 to: 1) do: [:xDiff |
		(-1 to: 1) do: [:yDiff | | val |
			val := self endlessBorderAt: x+xDiff at: y+yDiff.
			val ifNotNil: [
				(xDiff = 0 & yDiff = 0) ifFalse: [result add: val]]]].
	^ result! !

SudokuGUI subclass: #IrregularGridGUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:26'!
createCellsFrom: aMorph
	"This method creates all cells of an irregular grid. Each cell is like the given morph."
	| row grid |
	row := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		color: Color transparent;
		disableDragNDrop.
	9 timesRepeat: [row addMorph: aMorph copy].

	grid := RectangleMorph new
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		borderWidth: 0;
		layoutInset: 0;
		cellInset: 0;
		color: Color gray lighter;
		disableDragNDrop.
	9 timesRepeat: [grid addMorph: row copy].

	^ grid! !

!IrregularGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:25'!
initializeGrid 

	grid := self createCellsFrom: IrregularCell new.
	grid borderWidth: 0.! !

!IrregularGridGUI methodsFor: 'updating' stamp: 'mg 6/30/2014 13:53'!
setMatrixCoordinatesOf: anIrregularPuzzleGrid
	"Sets the x and y coordinates of the corresponding matrix."
	grid submorphs withIndexDo:
	[:morph :row | morph submorphs withIndexDo: 
		[:cell :column | 
			cell sudokuModel: anIrregularPuzzleGrid; xCoord: row; yCoord: column.
			sudokuCellDict at: row@column put: cell]].! !

IrregularGridGUI subclass: #StarbattleGridGUI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!StarbattleGridGUI methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:30'!
initializeGrid 

	grid := self createCellsFrom: StarbattleCell new.
	grid borderWidth: 0.! !

BorderedMorph subclass: #IrregularCell
	instanceVariableNames: 'sudokuModel xCoord yCoord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:42'!
addBottomBorder

	self borderStyle borderBottom: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addLeftBorder

	self borderStyle borderLeft: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addRightBorder

	self borderStyle borderRight: true.
	self changed.! !

!IrregularCell methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:43'!
addTopBorder

	self borderStyle borderTop: true.
	self changed.! !

!IrregularCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:40'!
handlesKeyboard: evt

	^ true! !

!IrregularCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 16:42'!
initialize

	| style |
	super initialize.
	style := IrregularGridBorderStyle new.
	style width: 1.
	style color: Color black.
	self 
		borderStyle: style;
		color: Color white;
		width: 77; height: 77.! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
sudokuModel

	^ sudokuModel! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
sudokuModel: anObject

	sudokuModel := anObject! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
xCoord

	^ xCoord! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
xCoord: anObject

	xCoord := anObject! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
yCoord

	^ yCoord! !

!IrregularCell methodsFor: 'accessing' stamp: 'mg 6/30/2014 13:24'!
yCoord: anObject

	yCoord := anObject! !

IrregularCell subclass: #StarbattleCell
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!StarbattleCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:52'!
addImageMorphForFile: anImageFile

	image := ImageMorph new image: (Form fromFileNamed: anImageFile).
	image image: (image image scaledToSize: self extent).
	self addMorph: image.
	image hide.! !

!StarbattleCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:50'!
handlesMouseDown: anEvent

	^ true! !

!StarbattleCell methodsFor: 'initialization' stamp: 'mg 6/30/2014 17:28'!
initialize

	super initialize.
	self
		layoutPolicy: TableLayout new;
		listCentering: #center;
		wrapCentering: #center;
		addImageMorphForFile: 'star.png'.! !

!StarbattleCell methodsFor: 'event handling' stamp: 'mg 6/30/2014 17:53'!
mouseDown: evt

	image visible 
		ifTrue: [image hide]
		ifFalse: [image show].! !

SimpleBorder subclass: #IrregularGridBorderStyle
	instanceVariableNames: 'borderTop borderBottom borderLeft borderRight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Puzzles-GUI'!

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderBottom

	^ borderBottom! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderBottom: anObject

	borderBottom := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderLeft

	^ borderLeft! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderLeft: anObject

	borderLeft := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderRight

	^ borderRight! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderRight: anObject

	borderRight := anObject! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderTop

	^ borderTop! !

!IrregularGridBorderStyle methodsFor: 'accessing' stamp: 'mg 6/30/2014 16:34'!
borderTop: anObject

	borderTop := anObject! !

!IrregularGridBorderStyle methodsFor: 'drawing' stamp: 'mg 6/30/2014 16:48'!
frameRectangle: aRectangle on: aCanvas
	"Borders are drawn with greater borderWidth."
	aCanvas frameAndFillRectangle: aRectangle
		fillColor: Color transparent
		borderWidth: 1
		topLeftColor: Color gray lighter
		bottomRightColor: Color gray.

	self borderTop 
		ifTrue: [aCanvas line: aRectangle topLeft + (2@2) to: aRectangle topRight + (-2@2) width: 4 color: Color black].
	self borderBottom
		ifTrue: [aCanvas line: aRectangle bottomLeft + (2@(-2)) to: aRectangle bottomRight + ((-2)@(-2)) width: 4 color: Color black].
	self borderLeft
		ifTrue: [aCanvas line: aRectangle topLeft + (2@2) to: aRectangle bottomLeft + (2@(-2)) width: 4 color: Color black].
	self borderRight
		ifTrue: [aCanvas line: aRectangle topRight + ((-2)@2) to: aRectangle bottomRight + ((-2)@(-2)) width: 4 color: Color black].! !

!IrregularGridBorderStyle methodsFor: 'initialize-release' stamp: 'mg 6/30/2014 16:35'!
initialize

	super initialize.
	self borderTop: false.
	self borderBottom: false.
	self borderLeft: false.
	self borderRight: false.! !
